package BasePopulation;

#   The PerlGP Genetic Programming System
#   Copyright (C) 2003 Bob MacCallum
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License ($PERLGP_BASE/LICENSE) for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   The original author's contact details are:
#   Bob MacCallum
#   Stockholm Bioinformatics Center
#   Stockholm University, 106 91 Stockholm, Sweden.
#   WWW: http://perlgp.org  EMAIL: info AT perlgp DOT org


use PerlGPObject;

@ISA = qw(PerlGPObject);

my %usedfilestems;

sub _init {
  my ($self, %p) = @_;

  my %defaults = ( PopulationSize => 2000,
		   Individuals => [ ],
		 );

  $self->SUPER::_init(%defaults, %p);

  $self->compulsoryParams(qw(ExperimentId));
}

sub countIndividuals {
  my $self = shift;
  return scalar @{$self->{Individuals}};
}

sub addIndividual {
  my ($self, $indiv) = @_;
  push @{$self->{Individuals}}, $indiv;
}


sub randomIndividual {
  my $self = shift;
  return $self->{Individuals}->[int rand @{$self->{Individuals}}];
}

sub selectCohort {
  my ($self, $size) = @_;
  my (%seen, @cohort);
  my $psize = $self->countIndividuals();
  while (keys %seen < $size && keys %seen < $psize) {
    my $ind = $self->randomIndividual();
    push @cohort, $ind unless ($seen{$ind}++); # make sure no duplicates
  }
  return @cohort;
}


1;
